require "spec_helper"

RSpec.describe "Employees" do
  before do
    @client = Bamboozled.client(subdomain: "x", api_key: "x")
  end

  it "Gets all employees" do
    response = File.new("spec/fixtures/all_employees.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    employees = @client.employee.all

    expect(employees).to be_a Array
    expect(employees.first.count).to eq 7
  end

  it "Gets one employee" do
    response = File.new("spec/fixtures/one_employee.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    employee = @client.employee.find(1234)

    expect(employee).to be_a Hash
    expect(employee.count).to eq 3
    expect(employee["firstName"]).to eq "John"
    expect(employee["lastName"]).to eq "Doe"
  end

  it "Gets employee job info" do
    response = File.new("spec/fixtures/job_info.xml")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    info = @client.employee.job_info(1234)

    expect(info).to be_a Hash
    expect(info[:table][:row].first[:employeeId]).to eq "100"

    info[:table][:row].first[:field].each do |f|
      case f[:id]
      when "location"
        expect(f[:__content__]).to eq "New York Office"
      when "division"
        expect(f[:__content__]).to eq "Sprockets"
      when "department"
        expect(f[:__content__]).to eq "Research and Development"
      when "jobTitle"
        expect(f[:__content__]).to eq "Machinist"
      when "reportsTo"
        expect(f[:__content__]).to eq "John Smith"
      end
    end
  end

  it "Gets an employee's time off estimate" do
    response = File.new("spec/fixtures/time_off_estimate.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    future = Time.now + (60 * 60 * 24 * 180)
    estimate = @client.employee.time_off_estimate(1234, future)

    expect(estimate).to be_a Hash
    expect(estimate["estimates"].keys).to match_array %w(end estimate)
    expect(estimate["estimates"]["estimate"].count).to eq 2
    expect(estimate["estimates"]["estimate"].first.keys)
      .to match_array %w(timeOffType name units balance)
  end

  it "Adjusts an employee's time off balance" do
    response = File.new("spec/fixtures/time_off_balance_adjustment.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    options = {
      date: "2014-06-17",
      timeOffTypeId: 2,
      amount: -1.5,
      note: "test"
    }

    result = @client.employee.time_off_balance_adjustment("44259", options)
    expect(result).to be_a Hash
  end

  it "Gets an employee's time off policies" do
    response = File.new("spec/fixtures/employee_time_off_policies.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    policies = @client.employee.time_off_policies("44259")

    expect(policies).to be_a Array
    expect(policies.first.keys).to match_array %w(timeOffPolicyId timeOffTypeId accrualStartDate)
  end

  it "adds an employee time off policy" do
    response = File.new("spec/fixtures/employee_time_off_policies.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    policies = [{
      timeOffPolicyId: 2,
      accrualStartDate: "2014-06-17"
    }]

    result = @client.employee.add_time_off_policies("44259", policies)
    expect(policies).to be_a Array
    expect(result.first.keys).to match_array %w(timeOffPolicyId timeOffTypeId accrualStartDate)
  end

  it "returns the proper url using employee email address" do
    hashed = "4fdce145bab6d27d69e34403f99fd11c" # Hash of me@here.com
    required_url = "http://x.bamboohr.com/employees/photos/?h=#{hashed}"

    # Normal
    url = @client.employee.photo_url("me@here.com")
    expect(url).to eq required_url

    # Email with spaces
    url = @client.employee.photo_url(" me@here.com ")
    expect(url).to eq required_url

    # Uppercase emails
    url = @client.employee.photo_url("ME@HERE.COM")
    expect(url).to eq required_url
  end

  it "returns the proper url using employee id" do
    response = File.new("spec/fixtures/employee_emails.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    hashed = "4fdce145bab6d27d69e34403f99fd11c"
    required_url = "http://x.bamboohr.com/employees/photos/?h=#{hashed}"

    url = @client.employee.photo_url(123)
    expect(url).to eq required_url
  end

  it "Gets all employee records which have changed since a given date" do
    response = File.new("spec/fixtures/last_changed.json")
    stub_request(:any, /.*api\.bamboohr\.com.*/).to_return(response)

    employees = @client.employee.last_changed("2011-06-02T19:26:23+00:00")

    expect(employees).to be_a Hash
    expect(employees.keys.count).to eq 4
  end

  describe "#add" do
    it "creates a new employee in BambooHR" do
      xml = YAML.load_file("spec/fixtures/add_employee_xml.yml")
      response = File.new("spec/fixtures/add_employee_response.json")
      details = JSON.parse(File.read("spec/fixtures/add_employee_details.json"))

      stub_request(:post, /.*api\.bamboohr\.com.*/)
        .with(xml).to_return(response)

      employee = @client.employee.add(details)
      location = employee["headers"]["location"]

      expect(location).to eq "https://api.bamboohr.com/api/gateway.php/alphasights/v1/employees/44259"
    end
  end

  describe "#update" do
    it "updates an employee in BambooHR" do
      xml = YAML.load_file("spec/fixtures/update_employee_xml.yml")
      response = File.new("spec/fixtures/update_employee_response.json")
      details = JSON.parse(File.read("spec/fixtures/update_employee_details.json"))
      url = "https://x:x@api.bamboohr.com/api/gateway.php/x/v1/employees/1234"

      stub_request(:post, url).with(xml).to_return(response)
      employee = @client.employee.update("1234", details)
      expected_headers = {
        "content-type" => ["application/json; charset=utf-8"],
        "date"         => ["Tue, 17 Jun 2014 19:25:35 UTC"]
      }

      expect(employee["headers"]).to eq(expected_headers)
    end
  end

  describe "#table_data" do
    it "Gets the Employee table data" do
      response = File.new("spec/fixtures/employee_table_details.json")
      stub_request(:get, /.*api\.bamboohr\.com.*/).to_return(response)

      employee_data = @client.employee.table_data(1, "jobInfo")
      expect(employee_data).to be_a Array
    end
  end

  describe "#add_table_row" do
    it "Adds a row to the employee table" do
      xml = YAML.load_file("spec/fixtures/update_employee_table_xml.yml")
      response = File.new("spec/fixtures/update_employee_response.json")
      details = JSON.parse(File.read("spec/fixtures/update_employee_table.json"))
      url = "https://x:x@api.bamboohr.com/api/gateway.php/x/v1/employees/1234/tables/jobInfo"

      stub_request(:post, url).with(xml).to_return(response)
      employee_table_row = @client.employee.add_table_row("1234", "jobInfo", details)
      expected_headers = {
        "content-type" => ["application/json; charset=utf-8"],
        "date"         => ["Tue, 17 Jun 2014 19:25:35 UTC"]
      }

      expect(employee_table_row["headers"]).to eq(expected_headers)
    end
  end

  describe "update_table_row" do
    it "updates the existing row to the table" do
      xml = YAML.load_file("spec/fixtures/update_employee_table_xml.yml")
      response = File.new("spec/fixtures/update_employee_response.json")
      details = JSON.parse(File.read("spec/fixtures/update_employee_table.json"))
      url = "https://x:x@api.bamboohr.com/api/gateway.php/x/v1/employees/1234/tables/jobInfo/1"

      stub_request(:post, url).with(xml).to_return(response)
      employee_table_row = @client.employee.update_table_row("1234", "jobInfo", 1, details)
      expected_headers = {
        "content-type" => ["application/json; charset=utf-8"],
        "date"         => ["Tue, 17 Jun 2014 19:25:35 UTC"]
      }

      expect(employee_table_row["headers"]).to eq(expected_headers)
    end
  end
end
